const spawn = require('child_process').spawn;

class Spawn {
  constructor(command) {

    this.subscribers = {}

    // linux
    this.child = spawn('/bin/sh',
      ['-c', command],
      {
        cwd: null,
        env: null,
        windowsVerbatimArguments: false
      })

    // TODO windows
    // this.child = require('child_process').spawn(
    //   'cmd.exe',
    //   ['/s', '/c', command],
    //   {
    //     cwd: null,
    //     env: null,
    //     windowsVerbatimArguments: true
    //   }
    // );


    this.child.stdout.on('data', (str) => {
      this.dispatch('data', '' + str)
    })

    this.child.stderr.on('data', (str) => {
      this.dispatch('data', '' + str)
    })

    this.child.on('error', (err) => {
      this.dispatch('error', `command exec error: ${err.message}`)
    })

    this.child.on('close', code => {
      if (code === 0) {
        this.dispatch('success', '')
      } else {
        this.dispatch('error', `command close error: ${code}`)
      }
    })

    this.pid = this.child.pid
  }


  /**
   * event name: 
   *  data, error, success
   */
  on(event, func) {
    this.subscribers[event] = func;
  }


  dispatch(event, args) {
    this.subscribers[event] && this.subscribers[event](args)
  }
}

module.exports = Spawn
